<?php
/*
OekakiPoteto 5.x Copyright 2000-2002 RanmaGuy (Theo Chakkapark, http://suteki.nu) and Marcello Bastéa-Forte (http://marcello.cellosoft.com). Modification to the files are permitted as long as my name remains on the files.

Wax Poteto modifications Copyright Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 5.9.0 - Last modified 2015-03-21
*/


// Load bootstrap
define('BOOT', 1);

// Config
$debug = false;
$update_ver = '1.5.5'; // Uses Wacintaki config/database version numbers
$update_db_ver = '1.5.5';
define('FORCE_MYSQL_API', 0); // 0=Auto, 1=MySQL, 2=MySQLi


// E_ALL still needs some more testing
error_reporting(E_ALL ^ E_NOTICE);


// Admin rank numbers
define('RANK_MOD',    4);
define('RANK_ADMIN',  5);
define('RANK_SADMIN', 7);
define('RANK_OWNER',  9);


// Init
$sql_errors = array();

$gd_OK = 0;
if (extension_loaded('gd')) {
	// GDlib available!
	$gd_OK = 1;
}

require 'common.php';

// Make db connection using new db_layer
define('PARSE_DBCONN_MANUALLY', 1);
$db_info = @parse_config_file('dbconn.php');
if ($db_info !== false) {
	$dbhost = $db_info['dbhost'];
	$dbuser = $db_info['dbuser'];
	$dbpass = $db_info['dbpass'];
	$dbname = $db_info['dbname'];
	$OekakiPoteto_MemberPrefix = $db_info['OekakiPoteto_MemberPrefix'];
	$OekakiPoteto_Prefix       = $db_info['OekakiPoteto_Prefix'];
	$db_mem = $OekakiPoteto_MemberPrefix;
	$db_p   = $OekakiPoteto_Prefix;

	require 'db_layer.php';
	db_open();
} else {
	print_footer();
	w_exit('STOP: Cannot read &ldquo;dbconn.php&drquo; file for database connection.');
}

// Import old config files
{
	$import_fail = 0;
	$err_write = array();
	$err_include = array();
	if (file_exists('dconn.php') && (!is_writable('dconn.php')) ) {
		$import_fail = 1;
		$err_write[] = 'dconn.php';
	}
	if (file_exists('config.php') && (!is_writable('config.php')) ) {
		$import_fail = 1;
		$err_write[] = 'config.php';
	}
	if (! @include 'config.php') {
		$import_fail = 1;
		$err_include[] = $file;
	}

	if ($import_fail > 0) {
echo <<<EOF

<hr />

<p>
	<strong>
		Warning:  update failed due to the following errors:<br />
		<p>

EOF;

		// Not writable
		if (count($err_write) > 0) {
			echo "\t\t\tFiles not writable:  ";
			web_line(make_list($err_write, ', ', '"'));
		}

		// Cannot include
		if (count($err_include) > 0) {
			echo "\t\t\tFiles cannot be included:  ";
			web_line(make_list($err_include, ', ', '"'));
		}

echo <<<EOF
		</p>
	</strong>
	Update cannot continue.<br />
	<br />
	Make sure your old board is working properly before running this updater.
</p>
EOF;

		print_footer();
		exit;
	}
}


// Determine oekaki version
{
	// OP 4
	$oe_version = '4.0.0';

	// OP 5
	if (isset($jpgcompression) && (!isset($wax_cfg_ver) || empty($wax_cfg_ver))) {
		$oe_version = '5.0.0';
	}

	// Wax up to 5.5.11
	if (isset($wax_cfg_ver)) {
		$oe_version = $wax_cfg_ver;
	}

	// Wax 5.6.0+
	if (isset($cfg['version'])) {
		$oe_version = $cfg['version'];
	}

	// Wax 5.6.4+ and Wac 1.3.0+
	// Only db_version matters, as it is updated with each cfg/db change
	if (isset($cfg['version'])) {
		$result = db_query("SELECT miscstring FROM {$db_p}oekakimisc WHERE miscname='dbversion'");

		if ($result && db_num_rows($result) > 0) {
			$oe_version = db_result($result);
		}
	}
}


// Mode
if (!isset($_GET['do'])) {
	if ($oe_version == $update_ver) {
		$mess1 = "Latest version of Wax detected";
		$mess2 = "You may <a href=\"index.php?action=secure\">safely remove the updater</a>.<br /><br /><a href=\"{$_SERVER['PHP_SELF']}?do=1\">Click here to verify the update</a>.";
	} elseif (isset($wax_cfg_ver)) {
		$mess1 = "Wax Poteto detected ({$oe_version})";
		$mess2 = "<a href=\"{$_SERVER['PHP_SELF']}?do=1\">Click here to start the update</a>.";
	} else {
		$mess1 = "Detected version of OekakiPoteto:  {$oe_version}";
		$mess2 = "<a href=\"{$_SERVER['PHP_SELF']}?do=1\">Click here to start the update</a>.";
	}
echo <<<EOF
<html>
<head>
	<title>OP to Wax Update</title>
</head>

<body>
	<h2>Wax Poteto updater: v{$update_ver}</h2>
	<p>
		{$mess1}
	</p>
	<p>
		{$mess2}
	</p>
</body>
</html>
EOF;
	exit;
}


// Prechecks
if (!is_writable('config.php')) {
	web_line('<strong>STOP</strong>:  &ldquo;config.php&rdquo; file is not writable.  CHMOD it before continuing.');
	fail_exit();
}
if (!is_writable('dbconn.php')) {
	web_line('<strong>STOP</strong>:  &ldquo;dbconn.php&rdquo; file is not writable.  CHMOD it before continuing.');
	fail_exit();
}
if ($oe_version == '4.0.0' && !file_exists('count')) {
	web_line('<strong>STOP</strong>:  &ldquo;count&rdquo; file cannot be found!  Check your board for corruption.');
	fail_exit();
}

// Detect apostrophes mode from old OP config file.
if (isset($apostrophes)) {
	if (!get_magic_quotes_gpc() ) {
		$apostrophes = '1';
	} else {
		$apostrophes = '0';
	}
}

// Fix messed up config settings
if (isset($BBStitle)) {
	$BBStitle = fix_cfg_quotes($BBStitle);
}



// -------------------------------------------------------------------
// BEGIN UPDATE ROUTINES
//
// Remember to always reset $oe_version and $oe_db_version after each
//   version block finishes updating
// -------------------------------------------------------------------

//-------------
// Update 4.0.0
// ------------
if ($oe_version == '4.0.0') {
	web_line('Update from OekakiPoteto 4.x to OekakiPoteto 5.x...');

	$mem = $OekakiPoteto_MemberPrefix;
	$pre = $OekakiPoteto_Prefix;

	// Lots of stuff to update
	run_q("ALTER TABLE {$pre}oekakichat RENAME {$mem}oekakichat");
	run_q("ALTER TABLE {$pre}oekakionline RENAME {$mem}oekakionline");
	run_q("ALTER TABLE {$pre}oekakikill RENAME {$mem}oekakikill");
	run_q("ALTER TABLE {$mem}oekakionline ADD onlineboard text");
	run_q("ALTER TABLE {$mem}oekaki CHANGE commcount commcount INT(11) DEFAULT 0 NOT NULL");
	run_q("ALTER TABLE {$mem}oekaki CHANGE piccount piccount INT(11) DEFAULT 0 NOT NULL");
	run_q("ALTER TABLE {$mem}oekakionline CHANGE OID OID BIGINT(255) NOT NULL AUTO_INCREMENT");
	run_q("ALTER TABLE {$mem}oekakionline CHANGE locale locale VARCHAR(30) DEFAULT NULL");
	run_q("ALTER TABLE {$pre}oekakidta CHANGE PIC_ID PIC_ID int(11) DEFAULT 0 NOT NULL");
	run_q("ALTER TABLE {$pre}oekakidta CHANGE edittime edittime int(20) unsigned default NULL");
	run_q("ALTER TABLE {$pre}oekakidta ADD postlock tinyint(4) NOT NULL default 0 ");
	run_q("ALTER TABLE {$mem}oekaki ADD age text");
	run_q("ALTER TABLE {$pre}oekakidta ADD animation tinyint(2) default 0");
	run_q("ALTER TABLE {$mem}oekaki ADD gender text");
	run_q("UPDATE {$pre}oekakidta SET postlock='1'");

	run_q("CREATE TABLE IF NOT EXISTS {$mem}oekakimailbox (MID int(11) NOT NULL auto_increment, sender varchar(255) NOT NULL default '', reciever varchar(255) NOT NULL default '', subject text, body longtext, mstatus smallint(10) NOT NULL default 0, senddate datetime NOT NULL default '0000-00-00 00:00:00', PRIMARY KEY (MID), KEY sender(sender), KEY reciever (reciever))");
	run_q("ALTER TABLE {$mem}oekaki ADD language text");
	run_q("UPDATE {$mem}oekaki SET language='english'");

	run_q("CREATE TABLE IF NOT EXISTS {$pre}oekakimisc (MiscID int(11) NOT NULL auto_increment, miscname varchar(255) NOT NULL default '', miscvalue bigint(255) NOT NULL DEFAULT 0, PRIMARY KEY (MiscID), KEY miscname (miscname))");

	$resno = 0;
	if (file_exists('count')) {
		$count_filesize = filesize('count');
		$fp2 = @fopen('count', 'r');
		if ($fp2) {
			$resno = (int) fread($fp2, $count_filesize);
			fclose($fp2);
		}
	}

	run_q("INSERT INTO {$pre}oekakimisc VALUES (1, 'piccount', $resno)");
	run_q("ALTER TABLE {$mem}oekaki ADD picview TINYINT(11) DEFAULT 0 NOT NULL");

	run_q("CREATE TABLE IF NOT EXISTS {$mem}oekakichatmod (usrname varchar(255) NOT NULL default '', command varchar(32) NOT NULL default 'MESSAGE', comment tinytext, posttime timestamp(14) NOT NULL, hostname text, email varchar(255) DEFAULT NULL, url text, IP text NOT NULL, postname varchar(50) NOT NULL default '', PRIMARY KEY (posttime)) TYPE=MyISAM;");

	run_q("CREATE TABLE IF NOT EXISTS {$mem}oekakichatmodusers (usrname varchar(255) NOT NULL DEFAULT '', lastmessage timestamp(14) NOT NULL, afk ENUM('no', 'yes') NOT NULL default 'no', PRIMARY KEY (usrname)) TYPE=MyISAM;");

	// Fail here before config is written
	$num_errs = count($sql_errors);
	if ($num_errs > 0) {
		for ($i = $num_errs; $i > 0; $i--) {
			web_line('MySQL: '.$sql_errors[$i - 1]);
		}
		web_line();
		web_line('<strong>STOP></strong>:  Update failed.  Check the database connection and run the updater again');
		fail_exit();
	}

	$fp = fopen('config.php', 'w');
	fwrite($fp, "<?php\n \$saltenc=\"{$saltenc}\"; \n \$OPpics=\"{$OPpics}\"; \n \$killuser=\"{$killuser}\"; \n \$template=\"{$template}\"; \n \$BBStitle=\"{$BBStitle}\"; \n \$eaddr=\"{$eaddr}\"; \n \$okurl=\"{$okurl}\"; \n \$apostrophes=\"{$apostrophes}\"; \n \$pstore=\"{$pstore}\"; \n \$ppage=\"{$ppage}\"; \n \$approval=\"{$approval}\"; \n \$drawaccess=\"{$drawaccess}\"; \n \$animationaccess=\"{$animationaccess}\"; \n \$xCanvas=\"{$xCanvas}\"; \n \$yCanvas=\"{$yCanvas}\"; \n \$xDefault=\"{$xDefault}\"; \n \$yDefault=\"{$yDefault}\"; \n \$chatMax=\"{$chatMax}\"; \n \$chatDisp=\"{$chatDisp}\"; \n \$language=\"english\"; \n \$archivedir=\"{$archivedir}\"; \n \$guestpost=\"{$guestpost}\";\n \$anisize=\"{$anisize}\"; \n \$jpgcompression=\"no\"; \n \$jpgcompressqual=\"100\";?>");
	fclose($fp);

	web_line('Update finished: OekakiPoteto 5.x');
	web_line();
	$oe_version = '5.0.0';
	$oe_db_ver = '';
}



// ------------
// Update 5.0.0
// ------------
if ($oe_version == '5.0.0') {
	web_line('Update from OekakiPoteto 5.x to Wax 5.5.6...');

	$mem = $OekakiPoteto_MemberPrefix;
	$pre = $OekakiPoteto_Prefix;

	// Fix for MySQL 5 (don't add stuff yet)
	run_q("ALTER TABLE {$mem}oekaki CHANGE templatesel templatesel varchar(255) NOT NULL DEFAULT ''");
	run_q("ALTER TABLE {$mem}oekakichat CHANGE comment comment TINYTEXT");
	run_q("ALTER TABLE {$mem}oekakichat CHANGE hostname hostname TEXT");
	run_q("ALTER TABLE {$mem}oekakichat CHANGE `IP` `IP` TEXT");
	run_q("ALTER TABLE {$mem}oekakionline CHANGE onlineIP onlineIP TEXT");
	run_q("ALTER TABLE {$mem}oekakionline CHANGE onlineboard onlineboard TEXT");
	run_q("ALTER TABLE {$mem}oekakimailbox CHANGE subject subject TEXT");
	run_q("ALTER TABLE {$mem}oekakimailbox CHANGE body body LONGTEXT");
	run_q("ALTER TABLE {$mem}oekakichatmod CHANGE comment comment TINYTEXT");
	run_q("ALTER TABLE {$mem}oekakichatmod CHANGE hostname hostname TEXT");
	run_q("ALTER TABLE {$mem}oekakichatmod CHANGE `IP` `IP` TEXT");
	run_q("ALTER TABLE {$mem}oekakichatmodusers CHANGE usrname usrname VARCHAR(255) NOT NULL DEFAULT ''");

	run_q("ALTER TABLE {$pre}oekakicmt CHANGE comment comment TEXT");
	run_q("ALTER TABLE {$pre}oekakicmt CHANGE hostname hostname TEXT");
	run_q("ALTER TABLE {$pre}oekakicmt CHANGE `IP` `IP` TEXT");
	run_q("ALTER TABLE {$pre}oekakidta CHANGE comment comment TEXT");
	run_q("ALTER TABLE {$pre}oekakidta CHANGE hostname hostname TEXT");
	run_q("ALTER TABLE {$pre}oekakidta CHANGE `IP` `IP` TEXT");

	// Add avatars folder
	if (! (isset($avatar_folder) && !empty($avatar_folder)) ) {
		// Not set
		$avatar_folder = 'avatars';
	}
	if (!file_exists($avatar_folder)) {
		if (! @mkdir($avatar_folder, 0777)) {
			web_line('<strong>STOP</strong>: Could not create &ldquo;'.$avatar_folder.'&rdquo; folder.  Check your server permissions.');
			fail_exit();
		}
	}
	if (!is_writable($avatar_folder)) {
		web_line('ERROR:  &ldquo;'.$avatar_folder.'&rdquo; folder is not writable.  CHMOD the folder to 755, 775, or 777 after the updater has finished.');
	}


	// Add to DB
	//$sql_errors = array();
	$fields = get_fields($mem.'oekaki');
	if (!in_array('avatar', $fields)) {
		run_q("ALTER TABLE {$mem}oekaki ADD avatar VARCHAR(128) NOT NULL DEFAULT ''");
	} elseif ($debug) {
		web_line('Avatar field OK');
	}

	// Fail here before config is written
	$num_errs = count($sql_errors);
	if ($num_errs > 0) {
		for ($i = $num_errs; $i > 0; $i--) {
			web_line('MySQL: '.$sql_errors[$i - 1]);
		}
		web_line();
		web_line('<strong>STOP</strong>:  Cannot add avatar field to database.  Check the database connection and run the updater again');
		fail_exit();
	}


	// Verify "patch" values for wax < 5.5.5
	if (!isset($scale)) {
		// In case not in config file yet.
		$scale = 0;
		$scale_pix = 150;

		if ($gd_OK) {
			// Default on.  Hey, why not?
			$scale = 2;
		}
	}
	if (!isset($uploadaccess)) {
		$uploadaccess = "no";
	}
	if (!isset($archivedir)) {
		$archivedir = "";
	}
	if (!isset($useChat)) {
		$useChat = "yes";
	}

	// Update config file
	$fp = fopen('config.php', 'w');
	$main_config = <<<EOF
<?php // Include only
	\$wax_cfg_ver = "5.5.6";
	\$saltenc = "{$saltenc}";
	\$OPpics = "{$OPpics}";
	\$killuser = "{$killuser}";
	\$template = "{$template}";
	\$BBStitle = "{$BBStitle}";
	\$eaddr = "{$eaddr}";
	\$okurl = "{$okurl}";
	\$apostrophes = "{$apostrophes}";
	\$pstore = "{$pstore}";
	\$ppage = "{$ppage}";
	\$approval = "{$approval}";
	\$drawaccess = "{$drawaccess}";
	\$animationaccess = "{$animationaccess}";
	\$uploadaccess = "{$uploadaccess}";
	\$xCanvas = "{$xCanvas}";
	\$yCanvas = "{$yCanvas}";
	\$xDefault = "{$xDefault}";
	\$yDefault = "{$yDefault}";
	\$useChat = "{$useChat}";
	\$chatMax = "{$chatMax}";
	\$chatDisp = "{$chatDisp}";
	\$language = "{$language}";
	\$archivedir = "{$archivedir}";
	\$guestpost = "{$guestpost}";
	\$anisize = "{$anisize}";
	\$jpgcompression = "{$jpgcompression}";
	\$jpgcompressqual = "{$jpgcompressqual}";
	\$scale = "{$scale}";
	\$scale_pix = "{$scale_pix}";
	\$use_avatars = "yes";
	\$use_c_avatars = "no";
	\$avatar_folder = "avatars";
	\$avatar_x = "60";
	\$avatar_y = "60";

EOF;
	fwrite($fp, $main_config, strlen($main_config));
	fclose($fp);

	// Remove files
	if (file_exists('redirect.php') && !$debug) {
		@unlink('redirect.php');
	}
	if (file_exists('arecover.php') && !$debug) {
		@unlink('arecover.php');
	}
	if (file_exists('flagchk.php')) {
		@unlink('flagchk.php');
	}

	web_line('Update finished: Wax 5.5.6');
	web_line();
	$oe_version = '5.5.6';
	$oe_db_ver = '';
}



// ---------------------------
// Update 5.5.6 (major update)
// ---------------------------
if ($oe_version == '5.5.6') {
	web_line('Update from Wax 5.5.6 to Wax 5.6.0...');

	// Flag for full update, set if new cfg fails
	$full_update = false;

	// Update config file if not updated already
	if (!isset($cfg['version']) && isset($wax_cfg_ver)) {
		$full_update = true;

		// Fix quotes (again)
		$BBStitle = addslashes(stripslashes($BBStitle));

		// Fix thumbnail options
		$cfg_set_use_thumb = ($gd_OK) ? '1' : '0';

		$fp = fopen('config.php', 'w');
		$main_config = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '5.6.0';

	\$cfg['op_title']  = '{$BBStitle}';
	\$cfg['op_author'] = ''; // WIP
	\$cfg['op_adult']  = 'no';
	\$cfg['op_email']  = '{$eaddr}';
	\$cfg['op_url']    = '{$okurl}';

	\$cfg['salt']    = '{$saltenc}';
	\$cfg['op_pics'] = '{$OPpics}';
	\$cfg['op_pre']  = '';

	\$cfg['kill_user']   = '{$killuser}';
	\$cfg['kill_reg']    = '15';
	\$cfg['approval']    = '{$approval}';
	\$cfg['draw_immune'] = '{$cfg['draw_immune']}';
	\$cfg['require_art'] = 'no';
	\$cfg['guest_post']  = '{$guestpost}';
	\$cfg['guest_draw']  = ''; //WIP
	\$cfg['guest_adult'] = ''; //WIP
	\$cfg['reg_draw']    = '{$drawaccess}';
	\$cfg['reg_anim']    = '{$animationaccess}';
	\$cfg['reg_upload']  = '{$uploadaccess}';
	\$cfg['reg_rules']   = 'no';

	\$cfg['language'] = '{$language}';
	\$cfg['template'] = '{$template}';
	\$cfg['tpl_path'] = 'templates';
	\$cfg['res_path'] = 'resource';

	\$cfg['pic_store']  = '{$pstore}';
	\$cfg['pic_pages']  = '{$ppage}';
	\$cfg['menu_pages'] = '20';
	\$cfg['pic_limit']      = ''; //WIP
	\$cfg['pic_limit_time'] = ''; //WIP

	\$cfg['porn_img']      = 'pr0n.png';
	\$cfg['preview_img']   = 'preview.png';
	\$cfg['preview_title'] = 'Preview';
	\$cfg['def_x']    = '{$xDefault}';
	\$cfg['def_y']    = '{$yDefault}';
	\$cfg['canvas_x'] = '{$xCanvas}';
	\$cfg['canvas_y'] = '{$yCanvas}';
	\$cfg['min_x']    = '50';
	\$cfg['min_y']    = '50';
	\$cfg['canvas_a'] = ''; // WIP
	\$cfg['max_anim'] = '{$anisize}';
	\$cfg['max_pic']  = '1000000';

	\$cfg['use_thumb']   = '{$cfg_set_use_thumb}';
	\$cfg['force_thumb'] = 'no';
	\$cfg['thumb_bytes'] = '100000';
	\$cfg['thumb_t']     = '120';
	\$cfg['thumb_r']     = '250';
	\$cfg['thumb_jpg']   = '{$jpgcompressqual}';

	\$cfg['use_mailbox'] = 'yes';

	\$cfg['use_chat']   = '{$useChat}';
	\$cfg['chat_max']   = '{$chatMax}';
	\$cfg['chat_pages'] = '{$chatDisp}';

	\$cfg['smilies'] = 'yes';

	\$cfg['use_avatars']   = 'yes';
	\$cfg['use_c_avatars'] = 'no';
	\$cfg['avatar_folder'] = 'avatars';
	\$cfg['avatar_x'] = '50';
	\$cfg['avatar_y'] = '50';

	\$cfg['public_retouch']   = 'no'; // WIP
	\$cfg['safety_storetime'] = '10'; // WIP
	\$cfg['safety_saves']     = 'no'; // WIP
	\$cfg['self_bump']        = 'no'; // WIP

EOF;
		fwrite($fp, $main_config, strlen($main_config));
		fclose($fp);
	}


	// Update database
	$mem = $OekakiPoteto_MemberPrefix;
	$pre = $OekakiPoteto_Prefix;

	if ($full_update) {
		// Fix default templates
		$result = db_query("ALTER TABLE {$mem}oekaki CHANGE templatesel templatesel VARCHAR(255) DEFAULT ''");

		// Dump oekakikill
		$result = db_query("DROP TABLE IF EXISTS {$mem}oekakikill");
	}


	// Get list of columns
	$result = db_query("SELECT * FROM {$mem}oekaki");
	$junk = db_fetch_array($result);


	// Add new database columns
	if (!isset($junk['ptype'])) {
		// Thumbnail mode for each user
		$result = db_query('ALTER TABLE '.$mem.'oekaki ADD thumbview TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Screen size for each user
		$result = db_query('ALTER TABLE '.$mem.'oekaki ADD screensize MEDIUMINT NOT NULL DEFAULT 800');
		sql_check($result);

		// Sets view mode for each user to automatic (0)
		$result = db_query('UPDATE '.$mem.'oekaki SET picview=0');
		sql_check($result);

		// Ability to lock threads.
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD threadlock TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Picture sizes
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD px MEDIUMINT DEFAULT 0');
		sql_check($result);
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD py MEDIUMINT DEFAULT 0');
		sql_check($result);

		// Yes, VARCHAR for datatype (file extension) is better than INT as it allows for file uploads, not just pictures.  It also makes things easier for future mods.
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD ptype VARCHAR(4)');
		sql_check($result);
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD ttype VARCHAR(4)');
		sql_check($result);
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD rtype VARCHAR(4)');
		sql_check($result);

		// Records if thumbnail is available (built), unavailable, or should not be used (dimmentions or filesize too small, etc).
		$result = db_query('ALTER TABLE '.$pre.'oekakidta ADD usethumb TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Add password field
		$result = db_query("ALTER TABLE {$pre}oekakidta ADD password text");
		sql_check($result);

		// Add origart field
		$result = db_query("ALTER TABLE {$pre}oekakidta ADD origart VARCHAR(255) NOT NULL DEFAULT ''");
		sql_check($result);

		// Add db version control
		$result = db_query("ALTER TABLE {$pre}oekakimisc ADD miscstring VARCHAR(14)");
		sql_check($result);
		$result = db_query("INSERT INTO {$pre}oekakimisc VALUES (2, 'dbversion', 0, '1.3.0')");
		sql_check($result);
	}


	// Set/reset defaults for changed features
	if ($full_update) {
		// Flush templates, due to change of DEFAULT
		$result = db_query("UPDATE {$mem}oekaki SET templatesel=''");

		// Layout display mode
		$result = db_query("UPDATE {$mem}oekaki SET picview='0'");

		// Thumbnail display mode
		$result = db_query("UPDATE {$mem}oekaki SET thumbview='0'");
	}


	// Add ranks if they don't exist
	if (!isset($junk['rank'])) {
		$result = db_query("ALTER TABLE {$mem}oekaki ADD rank TINYINT NOT NULL DEFAULT 0");

		// Set ranks
		$result_names = db_query("SELECT usrname, usrflags FROM {$mem}oekaki");
		$all_rows = db_num_rows($result_names);
		for ($i = 0; $i < $all_rows; $i++) {
			$rank = 0;

			$row = db_fetch_array($result_names);
			if (strstr($row['usrflags'], 'A'))
				$rank = RANK_ADMIN;
			if (strstr($row['usrflags'], 'S'))
				$rank = RANK_SADMIN;
			if (strstr($row['usrflags'], 'O'))
				$rank = RANK_OWNER;
			if ($rank > 0) {
				$result = db_query("UPDATE {$mem}oekaki SET rank={$rank} WHERE usrname='{$row['usrname']}'");
			}
		}

		// Update the admin flags so only ranks are used
		$result_names = db_query("SELECT usrname, usrflags FROM {$mem}oekaki");
		$all_rows = db_num_rows($result_names);
		for ($i = 0; $i < $all_rows; $i++) {
			$row = db_fetch_array($result_names);

			if (strstr($row['usrflags'], 'A')) {
				$new_flags = str_replace(array('O', 'S', 'A'), '', $row['usrflags']);

				$result = db_query("UPDATE {$mem}oekaki SET usrflags='$new_flags' WHERE usrname='{$row['usrname']}'");
			}
		}
	}


	// Fix member picture count
	$result = db_query("UPDATE {$mem}oekaki SET piccount=0 WHERE piccount < 0");


	// Build ptypes
	/*
	It's possible the source image can be a JPEG or PNG but there's no way to tell which is the primarty image unless we search the filesystem for both.  This is partly my fault thanks to my Wax Poteto upload script which allows people to upload JPEGs if $jpgcompression is enabled (normally, Poteto can only accept PNGs).

	If a PNG exists, we always use that, but if a JPEG exists exclusively, we use that instead.  Rather than bang the filesys, we'll read the directory and do lots of string searching to find out what's available.  There's probably a better way using stacks, but I'm stupid.  :)
	*/

	// Fix config file conversion
	if (!isset($OPpics)) {
		$OPpics = $cfg['op_pics'];
	}

	// Read pic folder.  Collect names of PNGs and JPEGs
	$handle = opendir($OPpics);
	while ($file = readdir($handle)) {
		if (substr($file, -4) == '.png' || substr($file, -4) == '.jpg') {
			// Image file.  If not a thumbnail, queue it.
			if (strpos($file, 't') === false) {
				$files[] = $file;
			}
		}
	}
	closedir($handle);

	// Collect database entries and let's sort things out
	$result = db_query('SELECT ID_2, PIC_ID, px, py, ptype FROM '.$pre.'oekakidta');
	$total_rows = db_num_rows($result);

	for ($i = 0; $i < $total_rows; $i++) {
		$row = db_fetch_array($result);

		// If ptype is occupied, we do nothing unless a force is needed.
		// Rather than juggle filetypes, assume valid extension is > 2 char.
		if (strlen($row['ptype']) < 2 || $force == true) {

			// Get the pic's datatype
			// Use PNG if avaiable, otherwise, use JPEG ("thumbnail" in Poteto)
			$my_pre = $row['PIC_ID'].'.';
			$found_img = false;

			if (in_array($my_pre.'png', $files)) {
				// Found PNG!
				$row['ptype'] = 'png';
				$found_img = true;
			} elseif (in_array($my_pre.'jpg', $files)) {
				// No PNG.  Found JPG!
				$row['ptype'] = 'jpg';
				$found_img = true;
			}

			// Verify.  Check ptype first to avoid filesys slowdown
			$image = $OPpics.'/'.$row['PIC_ID'].'.'.$row['ptype'];
			if ($found_img) {
				// Assume image may be corrupt
				$sizes = @GetImageSize($image);
				if (!$sizes) {
					if (isset($cfg['def_x'])) {
						$sizes = array($cfg['def_x'], $cfg['def_y']);
					} else {
						$sizes = array($xDefault, $yDefault);
					}
				}

				// OK, we've got all our info.  Insert it into DB
				if ($debug) {
					web_line("PIC: {$row['PIC_ID']}, <strong>X: {$sizes[0]}, Y: {$sizes[1]}</strong>, TYPE: {$row['ptype']}");
				} else {
					$result2 = db_query("UPDATE {$pre}oekakidta SET px={$sizes[0]}, py={$sizes[1]}, ptype='{$row['ptype']}' WHERE PIC_ID='{$row['PIC_ID']}'");
					sql_check($result2);
				}
			} else {
				// D'OH!  Picture missing!
				web_line('<strong>NOTE:</strong> missing image "'.$OPPrefix.$row['PIC_ID'].'.xxx" for post '.$row['ID_2'].'!  Consider deleting the post rather than trying to FTP a picture into the pictures folder.');
			}
		}
	}
	// End build ptypes


	// Update piccount for new sorting system (about time)
	$new_piccount = 0;
	$result = db_query("SELECT MAX(PIC_ID) FROM {$pre}oekakidta");
	if ($result)
		$new_piccount = (int) db_result($result);

	if ($new_piccount == 0) {
		if (isset($pstore)) {
			$new_piccount = $pstore;
		} else {
			$new_piccount = $cfg['pic_store'];
		}
	}

	$result = db_query("UPDATE {$pre}oekakimisc SET miscvalue={$new_piccount} WHERE miscname='piccount'");


	// Clean orphaned images, including unregisterd OP JPEGs, if any
	clean_images($debug);

	@unlink('compress.php');
	@unlink('flagchk.php');
	@unlink('MgLine.zip');
	@unlink('news.php');
	@unlink('noteBBSa.php');
	@unlink('oekakiBBSa.php');
	@unlink('online.php');
	@unlink('README.txt');
	@unlink('redirect.php');
	@unlink('template.php');
	@unlink('test.php');
	@unlink('tnew.php');
	@unlink('tnewa.php');
	@unlink('viewanio.php');
	@unlink('viewanis.php');
	@unlink('version.php');
	@unlink('versionchk.php');

	unset($full_update);
	unset($new_piccount);

	web_line('Update finished: Wax 5.6.0');
	web_line();
	$oe_version = '5.6.0';
	$oe_db_ver = '1.3.0';
}



// -------------------------------------
// Update 5.6.0 or 1.3.0 to 5.6.4/1.3.12
// -------------------------------------
if ($oe_version == '5.6.0' || $oe_version == '1.3.0') {
	web_line('Update/verify from Wax 5.6.0 to 5.6.4 (Wacintaki 1.3.12)...');

	$cfg = array();
	include 'config.php'; // We can do this more than once

	// Update config file if not updated already
	if ($cfg['version'] == '5.6.0') {

		// May contain apostrophes
		$my_title  = addslashes($cfg['op_title']);
		$my_author = addslashes($cfg['op_author']);
		$my_preview_title = addslashes($cfg['preview_title']);

		$main_configuration = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '1.3.0';

	\$cfg['op_title']  = '{$my_title}';
	\$cfg['op_author'] = '{$my_author}';
	\$cfg['op_adult']  = '{$cfg['op_adult']}';
	\$cfg['op_email']  = '{$cfg['op_email']}';
	\$cfg['op_url']    = '{$cfg['op_url']}';

	\$cfg['salt']    = '{$cfg['salt']}';
	\$cfg['op_pics'] = '{$cfg['op_pics']}';
	\$cfg['op_pre']  = '{$cfg['op_pre']}';

	\$cfg['kill_user']     = '{$cfg['kill_user']}';
	\$cfg['kill_reg']      = '{$cfg['kill_reg']}';
	\$cfg['approval']      = '{$cfg['approval']}';
	\$cfg['draw_immune']   = '{$cfg['draw_immune']}';
	\$cfg['require_art']   = '{$cfg['require_art']}';
	\$cfg['guest_post']    = '{$cfg['guest_post']}';
	\$cfg['guest_draw']    = ''; // WIP
	\$cfg['guest_adult']   = ''; // WIP
	\$cfg['humanity_post'] = '{$cfg['humanity_post']}';
	\$cfg['reg_draw']      = '{$cfg['reg_draw']}';
	\$cfg['reg_anim']      = '{$cfg['reg_anim']}';
	\$cfg['reg_upload']    = '{$cfg['reg_upload']}';
	\$cfg['reg_rules']     = '{$cfg['reg_rules']}';

	\$cfg['language'] = '{$cfg['language']}';
	\$cfg['template'] = '{$cfg['template']}';
	\$cfg['tpl_path'] = '{$cfg['tpl_path']}';
	\$cfg['res_path'] = '{$cfg['res_path']}';

	\$cfg['pic_store']  = '{$cfg['pic_store']}';
	\$cfg['pic_pages']  = '{$cfg['pic_pages']}';
	\$cfg['menu_pages'] = '{$cfg['menu_pages']}';
	\$cfg['pic_limit']      = ''; // WIP
	\$cfg['pic_limit_time'] = ''; // WIP

	\$cfg['porn_img']      = '{$cfg['porn_img']}';
	\$cfg['preview_img']   = '{$cfg['preview_img']}';
	\$cfg['preview_title'] = '{$my_preview_title}';
	\$cfg['def_x']    = '{$cfg['def_x']}';
	\$cfg['def_y']    = '{$cfg['def_y']}';
	\$cfg['canvas_x'] = '{$cfg['canvas_x']}';
	\$cfg['canvas_y'] = '{$cfg['canvas_y']}';
	\$cfg['min_x']    = '{$cfg['min_x']}';
	\$cfg['min_y']    = '{$cfg['min_y']}';
	\$cfg['canvas_a'] = ''; // WIP
	\$cfg['max_anim'] = '{$cfg['max_anim']}';
	\$cfg['max_pic']  = '{$cfg['max_pic']}';

	\$cfg['use_thumb']   = '{$cfg['use_thumb']}';
	\$cfg['force_thumb'] = '{$cfg['force_thumb']}';
	\$cfg['thumb_bytes'] = '{$cfg['thumb_bytes']}';
	\$cfg['thumb_t']     = '{$cfg['thumb_t']}';
	\$cfg['thumb_r']     = '{$cfg['thumb_r']}';
	\$cfg['thumb_jpg']   = '{$cfg['thumb_jpg']}';

	\$cfg['use_mailbox'] = '{$cfg['use_mailbox']}';

	\$cfg['use_chat']   = '{$cfg['use_chat']}';
	\$cfg['chat_max']   = '{$cfg['chat_max']}';
	\$cfg['chat_pages'] = '{$cfg['chat_pages']}';

	\$cfg['smilies'] = '{$cfg['smilies']}';

	\$cfg['use_avatars']   = '{$cfg['use_avatars']}';
	\$cfg['use_c_avatars'] = '{$cfg['use_c_avatars']}';
	\$cfg['avatar_folder'] = '{$cfg['avatar_folder']}';
	\$cfg['avatar_x'] = '{$cfg['avatar_x']}';
	\$cfg['avatar_y'] = '{$cfg['avatar_y']}';

	\$cfg['public_retouch']   = '{$cfg['public_retouch']}';
	\$cfg['safety_storetime'] = '{$cfg['safety_storetime']}';
	\$cfg['safety_saves']     = '{$cfg['safety_saves']}';
	\$cfg['self_bump']        = '{$cfg['self_bump']}';

EOF;

		// Write the config file
		if ($fp = fopen('config.php', 'w')) {
			fwrite($fp, $main_configuration);
			fclose($fp);
		} else {
			web_line('<strong>STOP</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			fail_exit();
		}
	}


	// Update database
	$pre = $OekakiPoteto_Prefix;
	$mem = $OekakiPoteto_MemberPrefix;

	// Fix member picture count(bug causing this finally fixed in 5.6.4)
	$result = db_query("UPDATE {$mem}oekaki SET piccount=0 WHERE piccount < 0");

	// Add new fields
	$q   = array();
	$q[] = "ALTER TABLE {$mem}oekaki ADD `group_id` MEDIUMINT DEFAULT 0";
	$q[] = "ALTER TABLE {$mem}oekaki ADD `email_show` TINYINT(3) DEFAULT 1";
	$q[] = "ALTER TABLE {$mem}oekaki ADD `smilies_show` TINYINT(3) DEFAULT 1";
	$q[] = "ALTER TABLE {$mem}oekaki ADD `timezone` FLOAT NOT NULL DEFAULT 0";
	$q[] = "ALTER TABLE {$mem}oekaki ADD `links` TEXT";

	$q[] = "CREATE TABLE IF NOT EXISTS {$mem}oekakilog (`ID` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT, `category` VARCHAR(255) NOT NULL DEFAULT '', `member` VARCHAR(255) NOT NULL DEFAULT '', `affected` VARCHAR(255), `value` TEXT, `board` VARCHAR(255) NOT NULL DEFAULT '', `date` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00', PRIMARY KEY (ID)) TYPE=MyISAM";

	foreach($q as $query) {
		$result = db_query($query);
		if (!$result) {
			web_line('MySQL: '.db_error());
		}
	}

	// Set DB version marker.
	// Due to an earlier fluke, some new 5.6.0 installs don't have the
	//   version marker, so check for it, first.
	$result = db_query("SELECT * FROM {$pre}oekakimisc");
	$rows = (int) db_num_rows($result);
	if ($result && $rows > 0) {
		if ($rows < 2) {
			// Hmm... no miscstring!  Add it.
			$result = db_query("ALTER TABLE {$pre}oekakimisc ADD miscstring VARCHAR(14)");
			$result = db_query("INSERT INTO {$pre}oekakimisc VALUES (2, 'dbversion', 0, '1.3.12')");
		} else {
			// Set DB to 1.3.12
			$result = db_query("UPDATE {$pre}oekakimisc SET miscstring='1.3.12' WHERE miscname='dbversion'");
		}
	} else {
		web_line('<strong>STOP</strong>: Could not verify Wax Poteto database version (miscstring)!  Try again.');
		fail_exit();
	}


	web_line('Update finished: Wacintaki 1.3.12');
	web_line();
	$oe_version = '1.3.12'; // Set to db version
	$oe_db_ver = '1.3.12';
}


// -------------------------------
// Update 5.6.4 or 1.3.12 to 1.5.5
// -------------------------------
if ($oe_version == '1.3.12' || $oe_version == '1.5.0') {
	web_line('Update/verify from Wax 5.6.4 to 5.8.2 (Wacintaki 1.5.5)...');

	// Update dbconn.php
	{
		$db_configuration = <<<EOF
<?php // Include only
	\$cfg_db = array();
	\$cfg_db['version'] = '1.5.0';

	\$dbhost = '{$dbhost}';
	\$dbuser = '{$dbuser}';
	\$dbpass = '{$dbpass}';
	\$dbname = '{$dbname}';

	\$OekakiPoteto_Prefix = '{$OekakiPoteto_Prefix}';
	\$OekakiPoteto_MemberPrefix = '{$OekakiPoteto_MemberPrefix}';

EOF;

		$fp = fopen('dbconn.php', 'w') or die('Could not open database config file "dbconn.php" for writing.  Check your server permissions');
			fwrite($fp, $db_configuration);
			fclose($fp);
	}


	$cfg = array();
	include 'config.php'; // We can do this more than once

	// Update config file if not updated already
	if ($cfg['version'] == '1.3.0' || $cfg['version'] == '1.5.0') {

		// May contain apostrophes
		$my_title  = addslashes($cfg['op_title']);
		$my_author = addslashes($cfg['op_author']);
		$my_preview_title = addslashes($cfg['preview_title']);

		$main_configuration = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '1.5.5';

	\$cfg['op_title']  = '{$my_title}';
	\$cfg['op_author'] = '{$my_author}';
	\$cfg['op_adult']  = '{$cfg['op_adult']}';
	\$cfg['op_email']  = '{$cfg['op_email']}';
	\$cfg['op_url']    = '{$cfg['op_url']}';
	\$cfg['cut_email'] = '0';

	\$cfg['salt']    = '{$cfg['salt']}';
	\$cfg['op_pics'] = '{$cfg['op_pics']}';
	\$cfg['op_pre']  = '{$cfg['op_pre']}';

	\$cfg['kill_user']     = '{$cfg['kill_user']}';
	\$cfg['kill_reg']      = '{$cfg['kill_reg']}';
	\$cfg['draw_immune']   = '{$cfg['draw_immune']}';
	\$cfg['private']       = 'no'; // WIP
	\$cfg['approval']      = '{$cfg['approval']}';
	\$cfg['require_art']   = '{$cfg['require_art']}';
	\$cfg['guest_post']    = '{$cfg['guest_post']}';
	\$cfg['guest_draw']    = ''; // WIP
	\$cfg['guest_adult']   = ''; // WIP
	\$cfg['humanity_post'] = '{$cfg['humanity_post']}';
	\$cfg['reg_draw']      = '{$cfg['reg_draw']}';
	\$cfg['reg_anim']      = '{$cfg['reg_anim']}';
	\$cfg['reg_upload']    = '{$cfg['reg_upload']}';
	\$cfg['reg_rules']     = '{$cfg['reg_rules']}';
	\$cfg['censoring']     = 'no'; // WIP

	\$cfg['language'] = '{$cfg['language']}';
	\$cfg['template'] = '{$cfg['template']}';
	\$cfg['tpl_path'] = '{$cfg['tpl_path']}';
	\$cfg['res_path'] = '{$cfg['res_path']}';

	\$cfg['pic_store']  = '{$cfg['pic_store']}';
	\$cfg['pic_pages']  = '{$cfg['pic_pages']}';
	\$cfg['menu_pages'] = '{$cfg['menu_pages']}';
	\$cfg['pic_limit']      = ''; // WIP
	\$cfg['pic_limit_time'] = ''; // WIP
	\$cfg['use_viewer'] = '0'; // WIP

	\$cfg['porn_img']      = '{$cfg['porn_img']}';
	\$cfg['preview_img']   = '{$cfg['preview_img']}';
	\$cfg['preview_title'] = '{$my_preview_title}';
	\$cfg['def_x']    = '{$cfg['def_x']}';
	\$cfg['def_y']    = '{$cfg['def_y']}';
	\$cfg['canvas_x'] = '{$cfg['canvas_x']}';
	\$cfg['canvas_y'] = '{$cfg['canvas_y']}';
	\$cfg['min_x']    = '{$cfg['min_x']}';
	\$cfg['min_y']    = '{$cfg['min_y']}';
	\$cfg['canvas_a'] = ''; // WIP
	\$cfg['max_anim'] = '{$cfg['max_anim']}';
	\$cfg['max_pic']  = '{$cfg['max_pic']}';

	\$cfg['use_thumb']   = '{$cfg['use_thumb']}';
	\$cfg['force_thumb'] = '{$cfg['force_thumb']}';
	\$cfg['thumb_bytes'] = '{$cfg['thumb_bytes']}';
	\$cfg['thumb_t']     = '{$cfg['thumb_t']}';
	\$cfg['thumb_r']     = '{$cfg['thumb_r']}';
	\$cfg['thumb_jpg']   = '{$cfg['thumb_jpg']}';

	\$cfg['use_mailbox'] = '{$cfg['use_mailbox']}';

	\$cfg['use_chat']   = '{$cfg['use_chat']}';
	\$cfg['chat_max']   = '{$cfg['chat_max']}';
	\$cfg['chat_pages'] = '{$cfg['chat_pages']}';

	\$cfg['smilies'] = '{$cfg['smilies']}';

	\$cfg['use_avatars']   = '{$cfg['use_avatars']}';
	\$cfg['use_c_avatars'] = '{$cfg['use_c_avatars']}';
	\$cfg['avatar_folder'] = '{$cfg['avatar_folder']}';
	\$cfg['avatar_x'] = '{$cfg['avatar_x']}';
	\$cfg['avatar_y'] = '{$cfg['avatar_y']}';

	\$cfg['public_retouch']   = 'no'; // WIP
	\$cfg['safety_saves']     = 'no'; // WIP
	\$cfg['safety_max']       = '2';  // WIP
	\$cfg['safety_storetime'] = '30'; // WIP
	\$cfg['self_bump']        = 'no'; // WIP

EOF;

		// Write the config file
		if ($fp = fopen('config.php', 'w')) {
			fwrite($fp, $main_configuration);
			fclose($fp);
		} else {
			web_line('<strong>STOP</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			fail_exit();
		}
	}


	// Update database
	$q   = array();
	$q[] = "ALTER TABLE {$db_mem}oekaki ADD `no_viewer` TINYINT(3) DEFAULT 0";
	$q[] = "ALTER TABLE {$db_mem}oekaki ADD `IP` TEXT DEFAULT NULL";
	$q[] = "UPDATE {$db_p}oekakimisc SET miscstring='1.5.5' WHERE miscname='dbversion'";

	foreach($q as $query) {
		$result = db_query($query);
		if (!$result) {
			web_line('MySQL: '.db_error());
		}
	}

	// Remove files
	@unlink('tedita.php');

	// Complete!
	web_line('Update finished: Wax 5.8.2 (Wacintaki 1.5.5)');
	web_line();
	$oe_version = '1.5.5';
	$oe_db_ver = '1.5.5';
}


// -----------------------
// DEFAULT: always do this
// -----------------------
// Remove files
if (!$debug) {
	if (file_exists('install.php'))
		@unlink('install.php');
	if (file_exists('update.php'))
		@unlink('update.php');
}


// -------------------------------------------------------------------
// END UPDATE ROUTINES
// -------------------------------------------------------------------



// All done!
db_close();

web_line('Update complete! If you have multiple boards, make sure each board is at the current version.');
web_line();
web_line('If using avatars with multiple boards, please read the manual (short version:  the master board uses <strong>avatars/</strong>, all other &ldquo;slave&rdquo; boards use <strong>../board1/avatars/</strong>)');
web_line();
web_line("<a href=\"index.php?action=secure\">Click here to remove the updater and continue to the BBS.</a>");

exit;



// -------------------------------------------------------------------
// Functions
// -------------------------------------------------------------------

// Forces file to be writable
// Returns FALSE bad input or missing file, 0 failure, 1 success
// INPUT: $filename, int CHMOD
function check_write($in, $mod = 755) {
	if ($mod == 0 || empty($in)) {
		// Bad input
		return false;
	}
	$mod = sprintf("%04d", $mod);

	if (!file_exists($in)) {
		return false;
	}

	if (!is_writable($in)) {
		if (! @chmod($in, $mod)) {
			return 0;
		}
	}
	return 1;
}

// get_dir_list: collects all files w/specific file extension(s)
// returns array(), FALSE
function get_dir_list($dir, $extensions = '') {
	$files = array();

	// PHP5 only
	if (empty($extensions) && function_exists('scandir')) {
		return scandir($dir);
	}

	$dh = opendir($dir);
	if (!$dh) return false;

	while (false !== ($filename = readdir($dh))) {
		if (empty($extensions)) {
			$files[] = $filename;
		} else {
			foreach($extensions as $ext) {
				if (strpos($filename, '.'.$ext) !== false) {
					$files[] = $filename;
				}
			}
		}
	}

	closedir($dh);
	return $files;
}

function run_q($query) {
	global $sql_errors, $debug;

	if ($debug) {
		web_line('DEBUG:  '.$query);
	}
	$result = db_query($query);
	if (!$result) {
		$sql_errors[] = db_error();
	} else {
		return $result;
	}
}

function fail_exit() {
	global $dbconn;

	db_close();
	web_line();
	web_line('If problems persist, try visiting the <a href="http://www.ninechime.com/forum/">NineChime Support Forum</a>.');
	exit;
}

function fix_cfg_quotes($in) {
	/*
		Strip ALL backslashes and replace with proper quotes.  Due to OP quirks, single quotes should be left alone.
		Consecutive backslashes are a big problem with OP 5.x.
		Note that html decoders are mostly PHP5, and not in all PHP release candidates, either.  Grr.
	*/

	$in = str_replace('\\', '', $in);
	$in = str_replace('"', '&quot;', $in);
	$in = addslashes($in);
	$in = str_replace("\\'", '\'', $in);

	return $in;
}

function get_fields($table) {
	global $dbconn;
	$names = array();

	$result = db_query("SHOW FIELDS FROM ".$table);
	if ($result) {
		$rows = db_num_rows($result);
		for ($i=0; $i < $rows; $i++) {
			$temp = db_fetch_row ($result);
			$names[] = $temp[0];
		}
		return $names;
	} else {
		return false;
	}
}

// clean_images():
// returns TRUE, or string error
function clean_images($debug = false) {
	global $dbconn, $db_mem, $db_p;

	include 'config.php';


	$extensions = array('png', 'gif', 'jpg', 'pch', 'oeb', 'tmp');
	$files      = array(); // Filenames
	$good_files = array(); // Flags for good files (TRUE/FALSE)
	$deleted_files = 0;


	// Get list of files, by matching types
	$files = get_dir_list($cfg['op_pics'], $extensions);

	if (!$files) return 'No matching files';

	// Set all file flags to bad
	// NOTE: $file_count used later
	$file_count = count($files);
	for ($i = 0; $i < $file_count; $i++) {
		$good_files[$i] = false;
	}

	if ($debug) {
		web_line('Started clean of orphaned files...');
	}

	// Get each row from DB, and check what's present
	$result = db_query("SELECT ID_2, PIC_ID, ptype, ttype, animation, datatype FROM {$db_p}oekakidta");
	if (!$result) return 'DB ('.db_error().')';

	$total_rows = db_num_rows($result);

	if ($debug) {
		web_line($file_count.' total files.');
		web_line($total_rows.' total rows in db.');
	}

	for ($i = 0; $i < $total_rows; $i++) {
		$row = db_fetch_array($result);

		// Build list of files to check
		$pic = array();
		if (!empty($row['ptype']))
			$pic[] = $row['PIC_ID'].'.'.$row['ptype'];
		if (!empty($row['ttype']))
			$pic[] = 't'.$row['PIC_ID'].'.'.$row['ttype'];
		if ($row['animation']) {
			if ($row['datatype'] == 0 || $row['datatype'] == 2 || $row['datatype'] == 3) {
				$pic[] = $row['PIC_ID'].'.pch';
			}
			if ($row['datatype'] == 1) {
				$pic[] = $row['PIC_ID'].'.oeb';
			}
		}

		// Search for files
		foreach($pic as $item) {
			// Each database item ($item)
			for ($j = 0; $j < $file_count; $j++) {
				// Each cached file ($files)
				if (strtolower($files[$j]) == strtolower($item)) {
					// Found file!
					$good_files[$j] = true;
					break;
				}
			}
		}
	}

	// Delete files without a good flag
	for ($i = 0; $i < $file_count; $i++) {
		if ($good_files[$i] === false) {
			// Bad file
			if (!$debug) {
				@unlink($cfg['op_pics'].'/'.$files[$i]);
				$deleted_files++;
			}
			if ($debug) {
				web_line($files[$i].': DELETE', true);
			}
		}
		if ($debug) {
			web_line($files[$i]);
		}
	}

	if ($deleted_files > 0 || $debug) {
		web_line("Cleaned up {$deleted_files} orphaned files.");
	}

	if ($debug) {
		web_line('Finished cleaning orphaned files.');
	}

	return true;
}

function parse_config_file($file) {
	// Returns keyed array of all key/val pairs in a config file.
	// Regex whitelist method, works on servers with "eval" disabled.

	// Normal vars
	$pattern_var1_val2 = 
		'!^\W*\$([A-Za-z0-9_]+)\s*=\s*[\'|"](.+)[\'|"];!';

	// Keyed vars
	$pattern_var1_key2_val3 =
		'!^\W*\$([A-Za-z0-9_]+)\[[\'|"]([A-Za-z0-9_]+)[\'|"]\]\s*=\s*[\'|"](.+)[\'|"];!';

	$parsed = array();
	if (file_exists($file) && filesize($file) > 0) {
		$handle = @fopen($file, 'r');
		if (!$handle) {
			trigger_error("parse_config_file: cannot open {$file} for reading", E_USER_NOTICE);
			return false;
		}

		while (!feof ($handle)) {
			$buffer = trim(fgets ($handle, 4096));

			if (preg_match($pattern_var1_val2, $buffer, $match)) {
				// Normal variable
				$parsed[$match[1]] = $match[2];
			}
			elseif (preg_match($pattern_var1_key2_val3, $buffer, $match)) {
				// Keyed variable
				$parsed[$match[1]][$match[2]] = $match[3];
			}
		}

		fclose($handle);
	} else {
		return false;
	}

	if (!empty($parsed)) {
		return $parsed;
	} else {
		return false;
	}
}